#include "stdafx.h"
#include "misc.h"
#include "ZipMax.h"
#include "ZipMaxDlg.h"


// CZipMaxApp

BEGIN_MESSAGE_MAP(CZipMaxApp, CWinApp)
END_MESSAGE_MAP()


// CZipMaxApp construction

CZipMaxApp::CZipMaxApp()
{
	// zipmax folder

	g_strMainFolder = "";


	// default window/column sizes/widths

	g_rectOutput = CRect(50, 50, 550, 270);
	g_nWidth0 = 280;
	g_nWidth1 = 78;
	g_nWidth2 = 130;


	// default zipmax.ini values

	for (int x = 0; x < MAXEXTERNAL; x++)
	{
		g_strPackerExe[x] = "";
		g_strCommandLine[x] = "";
	}

	g_strFinalExe = "";
	g_strFinalCommandLine = "";

	g_bNoBeep = FALSE;
	g_bSetCurrentTime = FALSE;

	g_bSubfolders = FALSE;
	g_bSkipIntern = FALSE;
	g_strPriority = "NORMAL";
	g_bThreadRunning = FALSE;
	g_bStop = FALSE;
	g_bLogFile = FALSE;
	g_bShowPackerWindow = FALSE;
	g_bSkipReadOnly = FALSE;
	g_bSkipArchive = FALSE; 
	g_bSetCurrentReadOnly = FALSE;

	ZeroMemory(&g_pi, sizeof(PROCESS_INFORMATION));
	g_bPause = FALSE;
	g_strOldStatus1 = "";
}


// The one and only CZipMaxApp object

CZipMaxApp theApp;


// CZipMaxApp initialization

BOOL CZipMaxApp::InitInstance()
{
	// InitCommonControls() is required on Windows XP if an application
	// manifest specifies use of ComCtl32.dll version 6 or later to enable
	// visual styles.  Otherwise, any window creation will fail.
	InitCommonControls();

	CWinApp::InitInstance();


	// check if zipmax is already running

	CString strTitle;
	strTitle.LoadString(IDS_STRING_TITLE);

	if (::FindWindow(NULL, strTitle) != NULL)
	{
		CString s;
		s.LoadString(IDS_ALREADYRUNNING);
		AfxMessageBox(s, MB_ICONWARNING | MB_OK);
		return FALSE;
	}

	// redefine application name (to get a real registry entry name)

	::free((void*)m_pszAppName);
	m_pszAppName=::_tcsdup(_T("ZipMax"));


	// get stored registry values (get default values from constructor)

	SetRegistryKey("");
	g_rectOutput.left = GetProfileInt("Windows", "Left", g_rectOutput.left);
	g_rectOutput.top = GetProfileInt("Windows", "Top", g_rectOutput.top);
	g_rectOutput.right = GetProfileInt("Windows", "Right", g_rectOutput.right);
	g_rectOutput.bottom = GetProfileInt("Windows", "Bottom", g_rectOutput.bottom);
	g_nWidth0 = GetProfileInt("Windows", "Width0", g_nWidth0);
	g_nWidth1 = GetProfileInt("Windows", "Width1", g_nWidth1);
	g_nWidth2 = GetProfileInt("Windows", "Width2", g_nWidth2);


	// get foldername of zipmax (and convert it to 8.3)
	// and switch to zipmax directory

	char szHomePath[_MAX_PATH];
	::GetModuleFileName(::AfxGetInstanceHandle(), szHomePath, _MAX_PATH);
	CString strPath(szHomePath);
	strPath = CMisc::GetPurePathName(strPath);
	char szBuffer[_MAX_PATH];
	::GetShortPathName(strPath, szBuffer, _MAX_PATH - 1);
	g_strMainFolder = szBuffer;
	if (g_strMainFolder.Right(1) != "\\")
		g_strMainFolder += '\\';

	SetCurrentDirectory((LPCTSTR)g_strMainFolder);


	// get values from zipmax.ini

	CString strIni,strNum;
	strIni.LoadString(IDS_INIFILE);
	strIni = g_strMainFolder + strIni;

	BOOL bKeyword1[MAXEXTERNAL], bKeyword2[MAXEXTERNAL];

	int nExtCount = 0;

	DWORD x = 0;
	while (nExtCount < MAXEXTERNAL)
	{
		bKeyword1[x] = bKeyword2[x] = FALSE;

		strNum.Format("-%u",nExtCount + 1);
		g_strPackerExe[x] = CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "PACKER-EXE" + strNum, &bKeyword1[x]);
		g_strCommandLine[x] = CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "PACKER-CMD" + strNum, &bKeyword2[x]);


		// if only PACKER-EXE is used (instead of PACKER-EXE-1), use it anyway
		if (x == 0 && g_strPackerExe[x] == "")
		{
			g_strPackerExe[x] = CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "PACKER-EXE", &bKeyword1[x]);
			g_strCommandLine[x] = CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "PACKER-CMD", &bKeyword2[x]);
		}

		if (g_strPackerExe[x] != "")
			x++;

		nExtCount++;
	}

	g_dwNumExtern = x;

	g_strFinalExe = CMisc::GetKeywordInFile(strIni,"[ZIPMAX]","FINALSTEP-EXE");
	g_strFinalCommandLine = CMisc::GetKeywordInFile(strIni,"[ZIPMAX]","FINALSTEP-CMD");

	g_strPriority = CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "THREAD-PRIORITY");
	g_bSubfolders = (CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "WORK-RECURSIVE") == "1");
	g_bSkipIntern = (CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "SKIP-INTERNZIP") == "1");
	g_bLogFile = (CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "LOGFILE") == "1");

	g_bNoBeep = (CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "NOBEEP") == "1");
	g_bSetCurrentTime = (CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "SETCURRENTTIME") == "1");

	g_bShowPackerWindow = (CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "SHOWPACKERWINDOW") == "1");

	g_bSkipReadOnly = (CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "SKIPREADONLY") == "1");
	g_bSkipArchive = (CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "SKIPARCHIVE") == "1"); 
	g_bSetCurrentReadOnly = (CMisc::GetKeywordInFile(strIni, "[ZIPMAX]", "SETCURRENTREADONLY") == "1");


	// check if the ini is valid

	CString q;
	q.LoadString(IDS_INVALIDINI);
	BOOL bOk = TRUE;

	if ((g_dwNumExtern < 1))
		bOk = FALSE;
	else
	{
		for (x = 0; x < g_dwNumExtern; x++)
		{
			if (! bKeyword1[x] || ! bKeyword2[x])
				break;	// no packer/cmd line entries

			if (g_strPackerExe[x] == "" || g_strCommandLine[x] == "")
				break;	// empty entries

			if (g_strCommandLine[x].Find("%1") == -1 || g_strCommandLine[x].Find("%2") == -1)
				break;	// no arguments
		}
	}

	if (x < g_dwNumExtern || ! bOk)
		AfxMessageBox(q + strIni, MB_ICONWARNING | MB_OK);
	else
	{
		BOOL bFoundPacker = TRUE;
	    CString strLastPacker = "";

		for (DWORD i = 0; i < g_dwNumExtern; i++)
		{
			if (strLastPacker.CompareNoCase(g_strPackerExe[i]))
			{
				// Not the same, check it if exists.
				if (! CMisc::FileExists(g_strPackerExe[i]))
				{
					bFoundPacker = FALSE;
					q.LoadString(IDS_CANTDETECT);
					AfxMessageBox(q + g_strPackerExe[i], MB_ICONWARNING | MB_OK);
				}
				strLastPacker = g_strPackerExe[i];
			}
		}

		if ((g_strFinalExe != "") && (!CMisc::FileExists(g_strFinalExe)))
			bFoundPacker = FALSE;

		if (bFoundPacker)
		{
			// open the main dialog....here we go...

			CZipMaxDlg dlg;
			m_pMainWnd = &dlg;
			dlg.DoModal();
		}
	}


	// write registry entries

	WriteProfileInt("Windows", "Left", g_rectOutput.left);
	WriteProfileInt("Windows", "Top", g_rectOutput.top);
	WriteProfileInt("Windows", "Right", g_rectOutput.right);
	WriteProfileInt("Windows", "Bottom", g_rectOutput.bottom);
	WriteProfileInt("Windows", "Width0", g_nWidth0);
	WriteProfileInt("Windows", "Width1", g_nWidth1);
	WriteProfileInt("Windows", "Width2", g_nWidth2);

	return FALSE;
}